/**********************************************************************
   Mark Butcher    Bsc (Hons) MPhil MIET

   M.J.Butcher Consulting
   Birchstrasse 20f,    CH-5406, Rtihof
   Switzerland

   www.uTasker.com    Skype: M_J_Butcher

   ---------------------------------------------------------------------
   File:        ADC_Timers.h
   Project:     uTasker Demonstration project
   ---------------------------------------------------------------------
   Copyright (C) M.J.Butcher Consulting 2004..2011
   *********************************************************************
   29.08.2009 Add timer frequency and PWM output tests                   {1}
   08.09.2009 Add INTERNAL_TEMP option for Luminary                      {2}
   07.11.2009 Add LPC2XXX PWM test and adjust PWM value to utilise macro {3}
   04.12.2009 Corrected syntax error                                     {4}
   14.01.2010 Add SAM7X PWM test                                         {5}
   03.03.2010 Add SAM7X capture mode test                                {6}
   13.04.2010 Add LPC17XX RIT test                                       {7}
   27.12.2010 Add SUPPORT_PWM_CONTROLLER mode for Luminary devices       {8}

The file is otherwise not specifically linked in to the project since it
is included by application.c when needed.

*/

#if !defined _ADC_TIMER_CONFIG
    #define _ADC_TIMER_CONFIG

    #if defined SUPPORT_ADC                                              // if HW support is enabled
    //#define TEST_ADC                                                   // enable test of ADC operation
    //#define MJB_AN7_TEST
    //#define MJB_AN7_TEST2
    //#define INTERNAL_TEMP                                              // {2} read also internal temperature (Luminary Micro)

        #if defined TEST_ADC && (defined _HW_SAM7X || defined _HW_AVR32) // SAM7X and AVR32 specific tests
          //#define _SAM7X_ADC_TEST1                                     // software controlled ADC triggering with an interrupt on each channel
          //#define _SAM7X_ADC_TEST2                                     // software controlled ADC triggering with a single interrupt on last channel completion
            #define _SAM7X_ADC_TEST3                                     // ADTRG input triggered conversion with a single interrupt on last channel completion
          //#define _SAM7X_ADC_TEST4                                     // TIOA2 output triggered conversion with a single interrupt on last channel completion
          //#define _SAM7X_ADC_TEST5                                     // TIOA2 output triggered conversion with DMA support saving to a sample buffer
          //#define _SAM7X_ADC_TEST6                                     // test zero crossing, high and low triggers
        #endif
    #endif
    #ifdef SUPPORT_PIT1                                                  // periodic interrupt
      //#define TEST_PIT                                                 // test a user defined periodic interrupt
      //#define TEST_PIT_SINGLE_SHOT
      //#define TEST_PIT_PERIODIC
    #endif
    #ifdef SUPPORT_DMA_TIMER                                             // M522XX DMA timers
      //#define TEST_DMA_TIMER                                           // test a user defined periodic interrupt
    #endif
    #ifdef SUPPORT_GENERAL_PURPOSE_TIMER                                 // general purpose timers
      //#define TEST_GPT                                                 // test general purpose timer operation
        #define GPT_CAPTURES     5                                       // when testing captures, collect this many values
    #endif
    #if defined SUPPORT_TIMER                                            // standard timers
        #define TEST_TIMER                                               // test a user defined timer interrupt
        #ifdef TEST_TIMER
          //#define TEST_SINGLE_SHOT_TIMER                               // test single-shot mode
          //#define TEST_PERIODIC_TIMER                                  // test periodic interrupt mode
          //#define TEST_ADC_TIMER                                       // test periodic ADC trigger mode (Luminary)
            #define TEST_PWM                                             // {1} test generating PWM output from timer
          //#define TEST_CAPTURE                                         // {6} test timer capture mode
        #endif
    #endif
    #ifdef SUPPORT_RIT
        #define RIT_TEST                                                 // {7} LPC17XX repetitive interrupt timer
    #endif

/* =================================================================== */
/*                 local function prototype declarations               */
/* =================================================================== */

    #ifdef TEST_ADC
        static void fnConfigureADC(void);
        #ifdef _M5223X
        static void adc_level_change_low(ADC_INTERRUPT_RESULT *adc_result);
        static void adc_level_change_high(ADC_INTERRUPT_RESULT *adc_result);
        #endif
    #endif
    #ifdef TEST_PIT
        static void fnConfigurePIT(void);
    #endif
    #ifdef TEST_DMA_TIMER
        static void fnConfigure_DMA_Timer(void);
    #endif
    #ifdef TEST_GPT
        static void fnConfigure_GPT(void);
    #endif
    #ifdef TEST_TIMER
        static void fnConfigure_Timer(void);
    #endif
    #if defined TEST_ADC && defined _SAM7X_ADC_TEST5
        static void fnRestartADC(ADC_SETUP *adc_setup);
    #endif
    #ifdef RIT_TEST
        static void fnConfigureRIT(void);
    #endif

/* =================================================================== */
/*                      local variable definitions                     */
/* =================================================================== */

    #if defined TEST_ADC
        #if defined _SAM7X_ADC_TEST5
            unsigned short *ptrADCValues = 0;
        #elif defined _LM3SXXXX
            #ifdef INTERNAL_TEMP
                #define ADC_CHANNELS_LM3S   (ADC_CHANNELS + 1)
            #else
                #define ADC_CHANNELS_LM3S   ADC_CHANNELS
            #endif
            #define ADC_SEQUENCES  2
            #define ADC_SAMPLES_LM3SXXXX (ADC_CHANNELS_LM3S * ADC_SEQUENCES)  // 2 x all channel sample sequences (3 with temperature)
            static unsigned short usADC_samples[ADC_SAMPLES_LM3SXXXX];
        #endif
    #endif

    #ifdef TEST_GPT
        static unsigned long ulCaptureList[GPT_CAPTURES];                // make space for capture values
    #endif
#endif


/* =================================================================== */
/*                      Initialisation code                            */
/* =================================================================== */
#ifdef _ADC_TIMER_INIT
    #ifdef TEST_ADC
        fnConfigureADC();
    #endif
    #ifdef TEST_PIT
        fnConfigurePIT();
    #endif
    #ifdef TEST_DMA_TIMER
        fnConfigure_DMA_Timer();
    #endif
    #ifdef TEST_GPT
        fnConfigure_GPT();
    #endif
    #ifdef TEST_TIMER
        fnConfigure_Timer();
    #endif
    #ifdef RIT_TEST
        fnConfigureRIT();
    #endif
#endif

#ifdef _ADC_TIMER_TIMER_EVENTS                                           // monostable timer event handling
#ifdef MJB_AN7_TEST
                else if (E_NEXT_SAMPLE == ucInputMessage[MSG_TIMER_EVENT]) {
                    ADC_SETUP adc_setup;                                 // interrupt configuration parameters
                    ADC_RESULTS results;
                    adc_setup.int_type = ADC_INTERRUPT;                  // identifier
                    adc_setup.int_priority = ADC_ERR_PRIORITY;           // port interrupt priority
                    adc_setup.int_adc_bit = 7;
                    adc_setup.int_adc_result = &results;

                    adc_setup.int_adc_mode = (ADC_GET_RESULT | ADC_READ_ONLY); // return the present result
                    fnConfigureInterrupt((void *)&adc_setup);            // enter interrupt for low level trigger
                    fnDebugDec(adc_setup.int_adc_bit, 7);
                    fnDebugHex(results.sADC_value[adc_setup.int_adc_bit], (WITH_SPACE | WITH_LEADIN | WITH_CR_LF | 2));
                    uTaskerMonoTimer( OWN_TASK, (DELAY_LIMIT)(3.0*SEC), E_NEXT_SAMPLE );
                }

#endif
    #if defined TEST_ADC && (defined _HW_SAM7X || defined _HW_AVR32)
            else if (E_NEXT_SAMPLE == ucInputMessage[MSG_TIMER_EVENT]) {
                ADC_SETUP adc_setup;                                     // interrupt configuration parameters
                adc_setup.int_type = ADC_INTERRUPT;                      // identifier when configuring
        #if defined _SAM7X_ADC_TEST5
                fnRestartADC(&adc_setup);
        #else
                adc_setup.int_adc_mode = (ADC_START_OPERATION);          // start next software conversion
        #endif
                fnConfigureInterrupt((void *)&adc_setup);                // configure test interrupt on high level trigger
            }
    #endif
    #if defined TEST_ADC && defined _LM3SXXXX
            else if (E_NEXT_SAMPLE == ucInputMessage[ MSG_TIMER_EVENT ]) {
                fnConfigureADC();                                        // repeat the ADC test
            }
    #endif
#endif

#if defined _ADC_TIMER_INT_EVENTS_1 && defined TEST_GPT                  // specific interrupt event handling
            case CAPTURE_COMPLETE_EVENT:
                {
                    int i = 0;
                    fnDebugMsg("Capture complete\r\n");
                    fnDebugMsg("================\r\n");
                    while (i < GPT_CAPTURES) {                           // list the capture samples
                        fnDebugHex(ulCaptureList[i], (WITH_LEADIN | WITH_CR_LF | sizeof(ulCaptureList[i])));
                        i++;
                    }
                }
                break;
#endif
#if defined _ADC_TIMER_INT_EVENTS_2 && defined TEST_ADC                  // interrupt range event handling
    #if defined _M5223X
                if ((ADC_LOW_7 >= ucInputMessage[MSG_INTERRUPT_EVENT]) && (ADC_HIGH_0 <= ucInputMessage[MSG_INTERRUPT_EVENT])) {
                    ADC_SETUP adc_setup;                                 // interrupt configuration parameters
                    ADC_RESULTS results;
                    adc_setup.int_type = ADC_INTERRUPT;                  // identifier
                    adc_setup.int_priority = ADC_ERR_PRIORITY;           // port interrupt priority
                    adc_setup.int_adc_result = &results;
                    if (ucInputMessage[MSG_INTERRUPT_EVENT] < ADC_LOW_0) {
                        adc_setup.int_adc_bit = (ucInputMessage[MSG_INTERRUPT_EVENT] - ADC_HIGH_0);                           // ADC bit 0
                        fnDebugMsg("ADC change high: ");
                        adc_setup.int_handler = adc_level_change_low;
                        adc_setup.int_adc_int_type = (ADC_LOW_LIMIT_INT);// interrupt types
                        adc_setup.int_low_level_trigger = (unsigned short)(ADC_VOLT * 1.5);
                    }
                    else  {
                        adc_setup.int_adc_bit = (ucInputMessage[MSG_INTERRUPT_EVENT] - ADC_LOW_0);
                        fnDebugMsg("ADC change low: ");
                        adc_setup.int_handler = adc_level_change_high;
                        adc_setup.int_high_level_trigger = (unsigned short)(ADC_VOLT * 2);
                        adc_setup.int_adc_int_type = (ADC_HIGH_LIMIT_INT);// interrupt types
                    }
                    adc_setup.int_adc_mode = (ADC_START_OPERATION | ADC_GET_RESULT);// start operation now and return the present result
                    fnConfigureInterrupt((void *)&adc_setup);            // enter interrupt for low level trigger
                    fnDebugDec(adc_setup.int_adc_bit, 0);
                    fnDebugHex(results.sADC_value[adc_setup.int_adc_bit], (WITH_SPACE | WITH_LEADIN | WITH_CR_LF | 2));
                }
                else if ((ADC_ZERO_CROSS_7 >= ucInputMessage[MSG_INTERRUPT_EVENT]) && (ADC_ZERO_CROSS_0 <= ucInputMessage[MSG_INTERRUPT_EVENT])) {
                    fnDebugMsg("Zero Crossing - channel ");
                    fnDebugDec((ucInputMessage[MSG_INTERRUPT_EVENT] - ADC_ZERO_CROSS_0), WITH_CR_LF);
                }
                else if (ADC_TRIGGER == ucInputMessage[MSG_INTERRUPT_EVENT]) {
                    ADC_SETUP adc_setup;                                 // interrupt configuration parameters
                    ADC_RESULTS results;
                    adc_setup.int_type = ADC_INTERRUPT;                  // identifier
                    adc_setup.int_adc_mode = (ADC_READ_ONLY | ADC_GET_RESULT);
                    adc_setup.int_adc_bit = 0;
                    adc_setup.int_adc_result = &results;
                    fnConfigureInterrupt((void *)&adc_setup);
                    fnDebugMsg("ADC triggered:");
                    fnDebugHex(results.sADC_value[0], (WITH_SPACE | WITH_LEADIN | WITH_CR_LF | sizeof(results.sADC_value[0]))); // {4}
                }
    #elif defined _HW_SAM7X || defined _HW_AVR32
                if (ADC_TRIGGER == ucInputMessage[MSG_INTERRUPT_EVENT]) {
        #if defined _SAM7X_ADC_TEST5
                    int iSamples;
                    unsigned short *ptrSamples = ptrADCValues;
                    fnDebugMsg("ADC results ready:\r\n");
                    fnDebugMsg("First 128 samples -\r\n");
                    for (iSamples = 0; iSamples < 128; iSamples++) {
                        fnDebugHex(*ptrSamples++, (WITH_SPACE | WITH_LEADIN | 2));
                        if (iSamples%8 == 7) {
                            fnDebugMsg("\r\n");
                        }
                    }
                    uTaskerMonoTimer( OWN_TASK, (DELAY_LIMIT)(3.0*SEC), E_NEXT_SAMPLE );
        #else
                    ADC_SETUP   adc_setup;                               // interrupt configuration parameters
                    ADC_RESULTS results;
                    int iChannel;
                    adc_setup.int_type = ADC_INTERRUPT;                  // identifier
                    adc_setup.int_adc_mode = (ADC_READ_ONLY | ADC_ALL_RESULTS);
                    adc_setup.int_adc_result = &results;
                    fnConfigureInterrupt((void *)&adc_setup);
                    fnDebugMsg("ADC results:\r\n");
                    for (iChannel = 0; iChannel < ADC_CHANNELS; iChannel++) {
                        fnDebugDec(iChannel, 0);
                        fnDebugMsg(": ");
                        fnDebugHex(results.sADC_value[iChannel], (WITH_SPACE | WITH_LEADIN | WITH_CR_LF | 2));
                    }
            #if defined _SAM7X_ADC_TEST1 || defined _SAM7X_ADC_TEST2
                    uTaskerMonoTimer( OWN_TASK, (DELAY_LIMIT)(1.0*SEC), E_NEXT_SAMPLE );
            #endif
                    fnDebugMsg("\n");
        #endif
                }
                else if ((ADC_TRIGGER_0 <= ucInputMessage[MSG_INTERRUPT_EVENT]) && (ADC_TRIGGER_7 >= ucInputMessage[MSG_INTERRUPT_EVENT])) {
                    ADC_SETUP   adc_setup;                               // interrupt configuration parameters
                    ADC_RESULTS results;
                    adc_setup.int_type = ADC_INTERRUPT;                  // identifier
                    adc_setup.int_adc_mode = (ADC_READ_ONLY | ADC_GET_RESULT);
                    adc_setup.int_adc_bits = (0x01 << (ucInputMessage[MSG_INTERRUPT_EVENT] - ADC_TRIGGER_0));
                    adc_setup.int_adc_result = &results;
                    fnConfigureInterrupt((void *)&adc_setup);
                    fnDebugMsg("ADC-");
                    fnDebugDec((ucInputMessage[MSG_INTERRUPT_EVENT] - ADC_TRIGGER_0), 0);
                    fnDebugMsg(":");
                    fnDebugHex(results.sADC_value[ucInputMessage[MSG_INTERRUPT_EVENT] - ADC_TRIGGER_0], (WITH_SPACE | WITH_LEADIN | WITH_CR_LF | 2));
                    uTaskerMonoTimer( OWN_TASK, (DELAY_LIMIT)(1.0*SEC), E_NEXT_SAMPLE );
                }
        #if defined _SAM7X_ADC_TEST6
                else if ((ADC_TRIGGER_5_ZERO <= ucInputMessage[MSG_INTERRUPT_EVENT]) && (ADC_TRIGGER_6_HIGH >= ucInputMessage[MSG_INTERRUPT_EVENT])) {
                    fnDebugMsg("ADC trigger channel ");
                    if ((ADC_TRIGGER_5_HIGH == ucInputMessage[MSG_INTERRUPT_EVENT]) || (ADC_TRIGGER_5_ZERO == ucInputMessage[MSG_INTERRUPT_EVENT]) || (ADC_TRIGGER_5_LOW == ucInputMessage[MSG_INTERRUPT_EVENT])) {
                        fnDebugMsg("5:");
                    }
                    else {
                        fnDebugMsg("6: ");
                        ucInputMessage[MSG_INTERRUPT_EVENT] -= 1;
                    }
                    switch (ucInputMessage[MSG_INTERRUPT_EVENT]) {
                    case ADC_TRIGGER_5_LOW:
                        fnDebugMsg("low trigger\r\n");
                        break;
                    case ADC_TRIGGER_5_HIGH:
                        fnDebugMsg("high trigger\r\n");
                        break;
                    case ADC_TRIGGER_5_ZERO:
                        fnDebugMsg("zero crossing trigger\r\n");
                        break;
                    }
                }
        #endif
    #elif defined _LM3SXXXX
                if (ADC_TRIGGER == ucInputMessage[MSG_INTERRUPT_EVENT]) {
                    int x;
                    fnDebugMsg("ADC Values:\r\n");
                    fnDebugMsg("===========");
                    for (x = 0; x < ADC_SAMPLES_LM3SXXXX; x++) {
                        fnDebugMsg("\r\nSample-");
                        fnDebugDec(x, 0);
                        fnDebugMsg(" = ");
                        fnDebugHex(usADC_samples[x], (WITH_LEADIN | sizeof(usADC_samples[x])));
                    }
                    fnDebugMsg("\r\n");
                    uTaskerMonoTimer( OWN_TASK, (DELAY_LIMIT)(5*SEC), E_NEXT_SAMPLE );
                }
    #endif
#endif



#if defined _ADC_TIMER_ROUTINES && defined TEST_ADC
    #if defined _HW_SAM7X || defined _HW_AVR32
        #ifdef _SAM7X_ADC_TEST1
static void adc_sample_complete(ADC_INTERRUPT_RESULT *adc_result)
{
    fnInterruptMessage(OWN_TASK, (unsigned char)(ADC_TRIGGER_0 + adc_result->ucADC_channel));
}
        #elif !defined _SAM7X_ADC_TEST6
static void adc_all_samples_complete(ADC_INTERRUPT_RESULT *adc_result)
{
    fnInterruptMessage(OWN_TASK, (unsigned char)ADC_TRIGGER);
}
        #endif
        #if defined _SAM7X_ADC_TEST6
static void adc_trigger(ADC_INTERRUPT_RESULT *adc_result)
{
    switch (adc_result->ucADC_channel) {
    case 5:
    case 6:
        if (adc_result->ucADC_flags & ADC_INT_ZERO_CROSSING) {
            fnInterruptMessage(OWN_TASK, (unsigned char)(ADC_TRIGGER_5_ZERO + (adc_result->ucADC_channel - 5)));
        }
        if (adc_result->ucADC_flags & ADC_INT_LOW_LEVEL) {
            fnInterruptMessage(OWN_TASK, (unsigned char)(ADC_TRIGGER_5_LOW  + (adc_result->ucADC_channel - 5)));
        }
        if (adc_result->ucADC_flags & ADC_INT_HIGH_LEVEL) {
            fnInterruptMessage(OWN_TASK, (unsigned char)(ADC_TRIGGER_5_HIGH + (adc_result->ucADC_channel - 5)));
        }
        break;
    }
}
        #endif

// ADC configuration for SAM7X / AVR32
//
static void fnConfigureADC(void)
{
        #ifdef _WINDOWS
            #define ADC_SAMPLES 10
        #else
            #define ADC_SAMPLES 1024
        #endif
    #define ADC_INPUTS  3
    ADC_SETUP adc_setup;                                                 // interrupt configuration parameters
    adc_setup.int_type = ADC_INTERRUPT;                                  // identifier when configuring
    adc_setup.int_priority = ADC_ERR_PRIORITY;                           // ADC interrupt priority
    adc_setup.int_adc_bits = (ADC_CHANNEL_0 | ADC_CHANNEL_5 | ADC_CHANNEL_6); // enable ADC bits 0, 5 and 6
    adc_setup.int_adc_result = 0;                                        // no result is requested
    adc_setup.int_adc_speed = (ADC_SAMPLING_SPEED(5000000));             // 5MHz sampling (5MHz is maximum for 10 bit mode and 8MHz maximum for 8 bit mode)

        #if defined _SAM7X_ADC_TEST1
    adc_setup.int_adc_int_type = (ADC_END_OF_SCAN_INT);                  // interrupt type - interrupt for each channel
    adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_SINGLE_SHOT_MODE | ADC_START_OPERATION); // single shot conversion, triggered immediately
    adc_setup.int_handler = adc_sample_complete;                         // handling function
        #elif defined _SAM7X_ADC_TEST2
    adc_setup.int_adc_int_type = (ADC_END_OF_SCAN_INT | ADC_SINGLE_SHOT_TRIGGER_INT); // interrupt type - single interrupt for last channel completed
    adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_SINGLE_SHOT_MODE | ADC_START_OPERATION); // single shot conversion, triggered immediately
    adc_setup.int_handler = adc_all_samples_complete;                    // handling function
        #elif defined _SAM7X_ADC_TEST3
    adc_setup.int_adc_int_type = (ADC_END_OF_SCAN_INT | ADC_SINGLE_SHOT_TRIGGER_INT); // interrupt type - single interrupt for last channel completed
    adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_SINGLE_SHOT_MODE | ADC_TRIGGERED_EXT); // single shot conversion, triggered immediately
    adc_setup.int_handler = adc_all_samples_complete;                    // handling function
        #elif defined _SAM7X_ADC_TEST4
    adc_setup.int_adc_int_type = (ADC_END_OF_SCAN_INT | ADC_SINGLE_SHOT_TRIGGER_INT); // interrupt type - single interrupt for last channel completed
    adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_LOOP_MODE | ADC_TRIGGERED_TIM2); // single shot conversion, triggered immediately
    adc_setup.int_handler = adc_all_samples_complete;                    // handling function
    adc_setup.int_adc_sample_rate = 1;                                   // 1Hz sampling rate - controlled by timer 2
        #elif defined _SAM7X_ADC_TEST5
    ptrADCValues = uMalloc(ADC_SAMPLES * 2 * ADC_INPUTS);                // space for sample values (in 10 bit mode each sample occupies 2 bytes)
    adc_setup.int_adc_int_type = (ADC_ENABLE_INTS);                      // interrupt type - interrupt on sample buffer full
    adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_LOOP_MODE | ADC_TRIGGERED_TIM2); // single shot conversion, triggered immediately
    adc_setup.int_handler = adc_all_samples_complete;                    // handling function
    adc_setup.int_adc_sample_rate = 1000;                                // 1kHz sampling rate - controlled by timer 2
    adc_setup.int_samples = (ADC_SAMPLES * ADC_INPUTS);                  // sample buffer size to be filled before interrupting
    adc_setup.int_adc_result = (void *)ptrADCValues;
        #elif defined _SAM7X_ADC_TEST6
    adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_LOOP_MODE | ADC_TRIGGERED_TIM2); // configure automatic trigger mode of operation
    adc_setup.int_zero_crossing_trigger = (unsigned short)(ADC_VOLT_10BIT * 1.65); // zero crossing value
    adc_setup.int_high_level_trigger = (unsigned short)(ADC_VOLT_10BIT * 2.5);// high trigger level
    adc_setup.int_low_level_trigger = (unsigned short)(ADC_VOLT_10BIT * 0.7); // low trigger level
    adc_setup.int_adc_sample_rate = 1;                                   // 1Hz sampling rate - controlled by timer 2
    adc_setup.int_adc_bits = (ADC_CHANNEL_5);                            // enable ADC bit 5
    adc_setup.int_handler = adc_trigger;                                 // handling function
    adc_setup.int_adc_int_type = (ADC_ZERO_CROSSING_INT_POSITIVE | ADC_ZERO_CROSSING_INT_NEGATIVE | ADC_LOW_LIMIT_INT | ADC_HIGH_LIMIT_INT | ADC_SINGLE_SHOT_TRIGGER_INT | ADC_SINGLE_SHOT_CROSSING_INT); // interrupt type - interrupt on and triggers but only once on each type
    fnConfigureInterrupt((void *)&adc_setup);                            // configure
    adc_setup.int_adc_mode = (ADC_CONFIGURE_CHANNEL | ADC_LOOP_MODE);    // add a trigger channel
    adc_setup.int_zero_crossing_trigger = (unsigned short)(ADC_VOLT_10BIT * 1.65); // zero crossing value
    adc_setup.int_high_level_trigger = (unsigned short)(ADC_VOLT_10BIT * 3.1);// high trigger level
    adc_setup.int_low_level_trigger = (unsigned short)(ADC_VOLT_10BIT * 0.35);// low trigger level
    adc_setup.int_adc_bits = (ADC_CHANNEL_6);                            // enable ADC bit 6
    adc_setup.int_adc_int_type = (ADC_ZERO_CROSSING_INT_POSITIVE | ADC_ZERO_CROSSING_INT_NEGATIVE | ADC_LOW_LIMIT_INT | ADC_HIGH_LIMIT_INT); // interrupt type - interrupt continuously
        #endif
    fnConfigureInterrupt((void *)&adc_setup);                            // configure
}
        #ifdef _SAM7X_ADC_TEST5
static void fnRestartADC(ADC_SETUP *adc_setup)
{
    adc_setup->int_adc_int_type = (ADC_ENABLE_INTS);                     // interrupt type - interrupt on sample buffer full
    adc_setup->int_priority = ADC_ERR_PRIORITY;                          // ADC interrupt priority
    adc_setup->int_adc_bits = (ADC_CHANNEL_0 | ADC_CHANNEL_5 | ADC_CHANNEL_6);// enable ADC bits 0, 5 and 6
    adc_setup->int_adc_mode = (ADC_CONFIGURE_CHANNEL | ADC_LOOP_MODE | ADC_TRIGGERED_TIM2); // single shot conversion, triggered immediately
    adc_setup->int_handler = adc_all_samples_complete;                   // handling function
    adc_setup->int_adc_sample_rate = 1000;                               // 1kHz sampling rate - controlled by timer 2
    adc_setup->int_samples = (ADC_SAMPLES * ADC_INPUTS);                 // sample buffer
    adc_setup->int_adc_result = (void *)ptrADCValues;
}
        #endif
    #elif defined _LM3SXXXX
static void adc_samples_ready(void)
{
    fnInterruptMessage(OWN_TASK, (unsigned char)ADC_TRIGGER);
}

static void fnConfigureADC(void)
{
    ADC_SETUP adc_setup;                                                 // interrupt configuration parameters
    adc_setup.int_type = ADC_INTERRUPT;                                  // identifier when configuring
    adc_setup.int_handler = adc_samples_ready;                           // handling function
    adc_setup.int_priority = PRIORITY_ADC;                               // ADC interrupt priority
        #if ADC_CHANNELS >= 8                                            // Luminary devices with 8 or more ADC inputs
  //adc_setup.int_adc_single_ended_inputs = (ADC_CHANNEL_0 | ADC_CHANNEL_1 | ADC_CHANNEL_2 | ADC_CHANNEL_3); // ADC channels 0..3 as single ended inputs 
    adc_setup.int_adc_single_ended_inputs = (ADC_CHANNEL_8 | ADC_CHANNEL_9 | ADC_CHANNEL_10 | ADC_CHANNEL_11); // ADC channels 0..3 as single ended inputs 
  //adc_setup.int_adc_differential_inputs = (ADC_CHANNEL_4 | ADC_CHANNEL_5 | ADC_CHANNEL_6 | ADC_CHANNEL_7); // ADC channels 4-5 and 6-7 as differential inputs
    adc_setup.int_adc_differential_inputs = (ADC_CHANNEL_12 | ADC_CHANNEL_13 | ADC_CHANNEL_14 | ADC_CHANNEL_15); // ADC channels 12-13 and 14-15 as differential inputs
        #else
            #ifdef INTERNAL_TEMP
    adc_setup.int_adc_single_ended_inputs = (ADC_CHANNEL_0 | ADC_CHANNEL_1 | ADC_CHANNEL_4_INTERNAL_TEMP);// ADC channels 0..1 as single ended inputs plus internal temperature
            #else
    adc_setup.int_adc_single_ended_inputs = (ADC_CHANNEL_0 | ADC_CHANNEL_1); // ADC channels 0..1 as single ended inputs
            #endif
    adc_setup.int_adc_differential_inputs = (ADC_CHANNEL_2 | ADC_CHANNEL_3); // ADC channels 2..3 as differential inputs
        #endif
        #ifdef TEST_ADC_TIMER
    adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_TRIGGER_TIMER);    // triggered from timer
        #else
  //adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_TRIGGER_GPIO_PB4_FALLING); // triggered by Port B-4 on a falling edge
    adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_START_OPERATION);  // immediate start under SW control
        #endif
    adc_setup.int_adc_averaging = HW_AVERAGING_64;                       // basic sampling speed is 1MHz but can be averaged to improve accuracy or reduce speed
  //adc_setup.int_adc_averaging = NO_HW_AVERAGING;                       // disable hardware averaging for highest speed
    adc_setup.int_adc_result = usADC_samples;                            // location to save the samples to
    adc_setup.int_sequence_count = ADC_SEQUENCES;
    fnConfigureInterrupt((void *)&adc_setup);                            // configure and start sequence
}
    #else                                                                // M5223X

// This interrupt is called when the ADC level changes above programmed threshold (on one of the enabled channels)
//
static void adc_level_change_high(ADC_INTERRUPT_RESULT *adc_result)
{
    if (!adc_result) {
        fnInterruptMessage(OWN_TASK, (unsigned char)(ADC_TRIGGER));
        return;
    }
    if (adc_result->ucADC_flags == ADC_INT_ZERO_CROSSING) {
        fnInterruptMessage(OWN_TASK, (unsigned char)(ADC_ZERO_CROSS_0 + adc_result->ucADC_channel));
    }
    else {
        fnInterruptMessage(OWN_TASK, (unsigned char)(ADC_HIGH_0 + adc_result->ucADC_channel));
    }
}

// this interrupt is called when the ADC level changes below programmed threshold (on one of the enabled channels)
//
static void adc_level_change_low(ADC_INTERRUPT_RESULT *adc_result)
{
    fnInterruptMessage(OWN_TASK, (unsigned char)(ADC_LOW_0 + adc_result->ucADC_channel));
}

#if defined MJB_AN7_TEST2

	#define VOLT_READ1     	MCF_GPIO_PORTAN = (unsigned char)((MCF_GPIO_PORTAN | MCF_GPIO_PORTAN_PORTAN3 | MCF_GPIO_PORTAN_PORTAN4 ) & ~(MCF_GPIO_PORTAN_PORTAN0 | MCF_GPIO_PORTAN_PORTAN1 | MCF_GPIO_PORTAN_PORTAN2))
	#define VOLT_READ2     	MCF_GPIO_PORTAN = (unsigned char)((MCF_GPIO_PORTAN | MCF_GPIO_PORTAN_PORTAN3 |  MCF_GPIO_PORTAN_PORTAN0 | MCF_GPIO_PORTAN_PORTAN4 ) & ~(MCF_GPIO_PORTAN_PORTAN1 | MCF_GPIO_PORTAN_PORTAN2 ))

	#define CURR_READ1     	MCF_GPIO_PORTAN = (unsigned char)(MCF_GPIO_PORTAN & ~(MCF_GPIO_PORTAN_PORTAN0 | MCF_GPIO_PORTAN_PORTAN1 | MCF_GPIO_PORTAN_PORTAN2 | MCF_GPIO_PORTAN_PORTAN3 | MCF_GPIO_PORTAN_PORTAN4))
	#define CURR_READ2     	MCF_GPIO_PORTAN = (unsigned char)((MCF_GPIO_PORTAN |  MCF_GPIO_PORTAN_PORTAN0) & ~(MCF_GPIO_PORTAN_PORTAN1 | MCF_GPIO_PORTAN_PORTAN2 | MCF_GPIO_PORTAN_PORTAN3 | MCF_GPIO_PORTAN_PORTAN4))

#define MCF_GPIO_PORTAN PORTAN

/* Bit definitions and macros for MCF_GPIO_PORTAN */
#define MCF_GPIO_PORTAN_PORTAN0              (0x1)
#define MCF_GPIO_PORTAN_PORTAN1              (0x2)
#define MCF_GPIO_PORTAN_PORTAN2              (0x4)
#define MCF_GPIO_PORTAN_PORTAN3              (0x8)
#define MCF_GPIO_PORTAN_PORTAN4              (0x10)
#define MCF_GPIO_PORTAN_PORTAN5              (0x20)
#define MCF_GPIO_PORTAN_PORTAN6              (0x40)
#define MCF_GPIO_PORTAN_PORTAN7              (0x80)

/*********************************************************************
* init_adc - Analog-to-Digital Converter (ADC)                       *
**********************************************************************/
#define MCF_ADC_ADLST2 ADC_ADLST2
#define MCF_ADC_ADLST2_SAMPLE7(x)            (((x)&0x7)<<0xC)
#define MCF_ADC_CTRL2 ADC_CTRL2
#define MCF_ADC_CTRL2_DIV(x)                 (((x)&0x1F)<<0)
#define MCF_ADC_CTRL2_SIMULT                 (0x20)
#define MCF_ADC_CTRL2_EOSIE1                 (0x800)
#define MCF_ADC_CTRL2_SYNC1                  (0x1000)
#define MCF_ADC_CTRL2_START1                 (0x2000)
#define MCF_ADC_CTRL2_STOP1                  (0x4000)

#define MCF_ADC_POWER  ADC_POWER
/* Bit definitions and macros for MCF_ADC_POWER */
#define MCF_ADC_POWER_PD0                    (0x1)
#define MCF_ADC_POWER_PD1                    (0x2)
#define MCF_ADC_POWER_PD2                    (0x4)
#define MCF_ADC_POWER_APD                    (0x8)
#define MCF_ADC_POWER_PUDELAY(x)             (((x)&0x3F)<<0x4)
#define MCF_ADC_POWER_PSTS0                  (0x400)
#define MCF_ADC_POWER_PSTS1                  (0x800)
#define MCF_ADC_POWER_PSTS2                  (0x1000)
#define MCF_ADC_POWER_ASB                    (0x8000)


#define MCF_ADC_CTRL1  ADC_CTRL1

#define MCF_ADC_CTRL1_SMODE(x)               (((x)&0x7)<<0)
#define MCF_ADC_CTRL1_CHNCFG(x)              (((x)&0xF)<<0x4)
#define MCF_ADC_CTRL1_HLMTIE                 (0x100)
#define MCF_ADC_CTRL1_LLMTIE                 (0x200)
#define MCF_ADC_CTRL1_ZCIE                   (0x400)
#define MCF_ADC_CTRL1_EOSIE0                 (0x800)
#define MCF_ADC_CTRL1_SYNC0                  (0x1000)
#define MCF_ADC_CTRL1_START0                 (0x2000)
#define MCF_ADC_CTRL1_STOP0                  (0x4000)

#define MCF_GPIO_DDRAN  DDRAN

#define MCF_GPIO_DDRAN_DDRAN0                (0x1)
#define MCF_GPIO_DDRAN_DDRAN1                (0x2)
#define MCF_GPIO_DDRAN_DDRAN2                (0x4)
#define MCF_GPIO_DDRAN_DDRAN3                (0x8)
#define MCF_GPIO_DDRAN_DDRAN4                (0x10)
#define MCF_GPIO_DDRAN_DDRAN5                (0x20)
#define MCF_GPIO_DDRAN_DDRAN6                (0x40)
#define MCF_GPIO_DDRAN_DDRAN7                (0x80)

#define MCF_GPIO_PANPAR  PANPAR

/* Bit definitions and macros for MCF_GPIO_PANPAR */
#define MCF_GPIO_PANPAR_PANPAR0              (0x1)
#define MCF_GPIO_PANPAR_AN0_GPIO             (0)
#define MCF_GPIO_PANPAR_AN0_AN0              (0x1)
#define MCF_GPIO_PANPAR_PANPAR1              (0x2)
#define MCF_GPIO_PANPAR_AN1_GPIO             (0)
#define MCF_GPIO_PANPAR_AN1_AN1              (0x2)
#define MCF_GPIO_PANPAR_PANPAR2              (0x4)
#define MCF_GPIO_PANPAR_AN2_GPIO             (0)
#define MCF_GPIO_PANPAR_AN2_AN2              (0x4)
#define MCF_GPIO_PANPAR_PANPAR3              (0x8)
#define MCF_GPIO_PANPAR_AN3_GPIO             (0)
#define MCF_GPIO_PANPAR_AN3_AN3              (0x8)
#define MCF_GPIO_PANPAR_PANPAR4              (0x10)
#define MCF_GPIO_PANPAR_AN4_GPIO             (0)
#define MCF_GPIO_PANPAR_AN4_AN4              (0x10)
#define MCF_GPIO_PANPAR_PANPAR5              (0x20)
#define MCF_GPIO_PANPAR_AN5_GPIO             (0)
#define MCF_GPIO_PANPAR_AN5_AN5              (0x20)
#define MCF_GPIO_PANPAR_PANPAR6              (0x40)
#define MCF_GPIO_PANPAR_AN6_GPIO             (0)
#define MCF_GPIO_PANPAR_AN6_AN6              (0x40)
#define MCF_GPIO_PANPAR_PANPAR7              (0x80)
#define MCF_GPIO_PANPAR_AN7_GPIO             (0)
#define MCF_GPIO_PANPAR_AN7_AN7              (0x80)



#define MCF_ADC_ADSDIS ADC_ADSDIS

#define MCF_ADC_CAL ADC_CAL

/* Bit definitions and macros for MCF_ADC_ADSDIS */
#define MCF_ADC_ADSDIS_DS0                   (0x1)
#define MCF_ADC_ADSDIS_DS1                   (0x2)
#define MCF_ADC_ADSDIS_DS2                   (0x4)
#define MCF_ADC_ADSDIS_DS3                   (0x8)
#define MCF_ADC_ADSDIS_DS4                   (0x10)
#define MCF_ADC_ADSDIS_DS5                   (0x20)
#define MCF_ADC_ADSDIS_DS6                   (0x40)
#define MCF_ADC_ADSDIS_DS7                   (0x80)

#define MCF_ADC_ADSTAT  ADC_ADSTAT

/* Bit definitions and macros for MCF_ADC_ADSTAT */
#define MCF_ADC_ADSTAT_RDY0                  (0x1)
#define MCF_ADC_ADSTAT_RDY1                  (0x2)
#define MCF_ADC_ADSTAT_RDY2                  (0x4)
#define MCF_ADC_ADSTAT_RDY3                  (0x8)
#define MCF_ADC_ADSTAT_RDY4                  (0x10)
#define MCF_ADC_ADSTAT_RDY5                  (0x20)
#define MCF_ADC_ADSTAT_RDY6                  (0x40)
#define MCF_ADC_ADSTAT_RDY7                  (0x80)
#define MCF_ADC_ADSTAT_HLMTI                 (0x100)
#define MCF_ADC_ADSTAT_LLMTI                 (0x200)
#define MCF_ADC_ADSTAT_ZCI                   (0x400)
#define MCF_ADC_ADSTAT_EOSI0                 (0x800)
#define MCF_ADC_ADSTAT_EOSI1                 (0x1000)
#define MCF_ADC_ADSTAT_CIP1                  (0x4000)
#define MCF_ADC_ADSTAT_CIP0                  (0x8000)

#define MCF_ADC_ADRSLT7  ADC_ADRSLT7

int volt_buf1 = 0;
int volt_buf2 = 0;
int volt_buf3 = 0;

int volt1_accu = 0;
int volt2_accu = 0;
int volt3_accu = 0;

int curr_buf1 =0;
int curr_buf2 =0;
int curr_buf3 =0;
int curr_buf4 =0;
int curr_buf5 =0;

#define CURRENT_OFFSET 2048


int curr1_accu=0;
int curr2_accu=0;
int curr3_accu=0;
int curr4_accu=0;


int power1_accu = 0;
int power2_accu = 0;
int power3_accu = 0;
int power4_accu = 0;



static void test_adc(void) // once a ms
{
    			VOLT_READ1;
			MCF_ADC_CTRL1 = MCF_ADC_CTRL1_START0;

			while ((MCF_ADC_ADSTAT & (MCF_ADC_ADSTAT_CIP0 | MCF_ADC_ADSTAT_CIP1))) {}
			if ((MCF_ADC_ADSTAT  & MCF_ADC_ADSTAT_RDY7) ==  MCF_ADC_ADSTAT_RDY7)  
			{
    				CURR_READ1;
				volt_buf1 = (unsigned short)((MCF_ADC_ADRSLT7&0x7FF8)>>3) - 2048   ;
				MCF_ADC_CTRL1 = MCF_ADC_CTRL1_START0;
            			//if (measure_power_factor  % 350 == 0)
					//printf("V1B: %d\n", volt_buf1);	
    				volt1_accu += (volt_buf1 * volt_buf1);
			}

			while ((MCF_ADC_ADSTAT & (MCF_ADC_ADSTAT_CIP0 | MCF_ADC_ADSTAT_CIP1))) {}
			if ((MCF_ADC_ADSTAT  & MCF_ADC_ADSTAT_RDY7) ==  MCF_ADC_ADSTAT_RDY7)  
			{
     				CURR_READ2;
				curr_buf1 = (unsigned short) ((MCF_ADC_ADRSLT7&0x7FF8)>>3) - CURRENT_OFFSET ;			
				MCF_ADC_CTRL1 = MCF_ADC_CTRL1_START0;
            			//if (measure_power_factor  % 350 == 0)
					//printf("C1B: %d\n", curr_buf1);	
	   			curr1_accu += (curr_buf1 * curr_buf1);
				power1_accu += (volt_buf1 * curr_buf1);
			}


}





void init_adc (void)
{
    // Scan mode = Loop parallel, converters A and B run simultaneously
    // ADC clock frequency = 10.67 MHz
    // Voltage reference supplied by VDDA and VSSA
    // All ADC interrupts disabled
    //  
    // Sample list for converter A:
    //     Sample 0 : ANA0
    //                Low limit = $000, High limit = $FFF, Offset = $000
    //     Sample 1 : ANA1
    //                Low limit = $000, High limit = $FFF, Offset = $000
    //     Sample 2 : ANA2
    //                Low limit = $000, High limit = $FFF, Offset = $000
    //     Sample 3 : ANA3
    //                Low limit = $000, High limit = $FFF, Offset = $000
    //  
    // Sample list for converter B:
    //     Sample 4 : ANB0
    //                Low limit = $000, High limit = $FFF, Offset = $000
    //     Sample 5 : ANB1
    //                Low limit = $000, High limit = $FFF, Offset = $000
    //     Sample 6 : ANB2
    //                Low limit = $000, High limit = $FFF, Offset = $000
    //     Sample 7 : ANB3
    //                Low limit = $000, High limit = $FFF, Offset = $000


    // Initialise ADC

    //     CLST1[SAMPLE3]  = %011
    //     CLST1[SAMPLE2]  = %010
    //     CLST1[SAMPLE1]  = %001
    //     CLST1[SAMPLE0]  = 0 

    MCF_ADC_ADLST2 = MCF_ADC_ADLST2_SAMPLE7(0x7);

    //     CTRL2[STOP1]    = 1 
    //     CTRL2[START1]   = 0 
    //     CTRL2[SYNC1]    = 0 
    //     CTRL2[EOSIE1]   = 0 
    //     CTRL2[SIMULT]   = 1 
    //     CTRL2[DIV]      = $2
    //MCF_ADC_CTRL2 = MCF_ADC_CTRL2_DIV(3);
    //MCF_ADC_CTRL2 = MCF_ADC_CTRL2_DIV(9);
    //MCF_ADC_CTRL2 = MCF_ADC_CTRL2_DIV(14);   //Commented by Vasanta and added below
    //MCF_ADC_CTRL2 = MCF_ADC_CTRL2_DIV(3);
    //MCF_ADC_CTRL2 = MCF_ADC_CTRL2_DIV(2);
	MCF_ADC_CTRL2 = MCF_ADC_CTRL2_STOP1 | MCF_ADC_CTRL2_DIV(0x2);

    //MCF_ADC_CTRL2 = MCF_ADC_CTRL2_DIV(0x1f);

    // Power up ADC converter(s) in use

    //     PWR[ASB]        = 0 
    //     PWR[PUDELAY]    = $d
    //     PWR[APD]        = 0 
    //     PWR[PD2]        = 1 
    //     PWR[PD1]        = 0 
    //     PWR[PD0]        = 0 
    MCF_ADC_POWER = MCF_ADC_POWER_PUDELAY(4);

    //     CTRL1[STOP0]    = 1 
    //     CTRL1[START0]   = 0 
    //     CTRL1[SYNC0]    = 0 
    //     CTRL1[EOSIE0]   = 0 
    //     CTRL1[ZCIE]     = 0 
    //     CTRL1[LLMTIE]   = 0 
    //     CTRL1[HLMTIE]   = 0 
    //     CTRL1[CHNCFG3]  = 0 
    //     CTRL1[CHNCFG2]  = 0 
    //     CTRL1[CHNCFG1]  = 0 
    //     CTRL1[CHNCFG0]  = 0 
    //     CTRL1[SMODE]    = %010
	MCF_ADC_CTRL1 = MCF_ADC_CTRL1_SMODE(2);

    /* AN0-4 as GPIO Outputs, AN7 as ADC Input */

#if 0
    /* Pin assignments for port AN 
           Pin AN7 : Analog input AN7 
           Pin AN6 : Analog input AN6 
           Pin AN5 : Analog input AN5 
           Pin AN4 : Analog input AN4 
           Pin AN3 : Analog input AN3 
           Pin AN2 : Analog input AN2 
           Pin AN1 : Analog input AN1 
           Pin AN0 : Analog input AN0 
    */
    MCF_GPIO_DDRAN = 0;
    MCF_GPIO_PANPAR = MCF_GPIO_PANPAR_PANPAR7 |
                      MCF_GPIO_PANPAR_PANPAR6 |
                      MCF_GPIO_PANPAR_PANPAR5 |
                      MCF_GPIO_PANPAR_PANPAR4 |
                      MCF_GPIO_PANPAR_PANPAR3 |
                      MCF_GPIO_PANPAR_PANPAR2 |
                      MCF_GPIO_PANPAR_PANPAR1 |
                      MCF_GPIO_PANPAR_PANPAR0;    
#else

    /* Pin assignments for port AN 
           Pin AN7 : Analog input AN7 
           Pin AN6 : 
           Pin AN5 : 
           Pin AN4 : GPIO Input 
           Pin AN3 : GPIO Input 
           Pin AN2 : GPIO Input 
           Pin AN1 : GPIO Input 
           Pin AN0 : GPIO Input 
    */
    
	//PAN0-6 as GPIO o/p
    MCF_GPIO_DDRAN =  MCF_GPIO_DDRAN_DDRAN6 |
                      MCF_GPIO_DDRAN_DDRAN5 |
                      MCF_GPIO_DDRAN_DDRAN4 |
                      MCF_GPIO_DDRAN_DDRAN3 |
                      MCF_GPIO_DDRAN_DDRAN2 |
                      MCF_GPIO_DDRAN_DDRAN1 |
                      MCF_GPIO_DDRAN_DDRAN0;    

    //AN7 as ADC INput
    MCF_GPIO_PANPAR = MCF_GPIO_PANPAR_PANPAR7; 
    //MCF_GPIO_PANPAR = 0;

    MCF_GPIO_PORTAN = 0;

    /* Pin assignments for port NQ 
           Pins NQ7-NQ1 : EdgePort GPIO/IRQ 
    */
    
    //MCF_GPIO_PORTNQ = 0;
/*
    MCF_GPIO_DDRNQ = MCF_GPIO_PNQPAR_PNQPAR4(0x1) |
    			MCF_GPIO_PNQPAR_PNQPAR7(0x1);

    MCF_GPIO_PNQPAR = MCF_GPIO_PNQPAR_PNQPAR6(0x1) |
                      MCF_GPIO_PNQPAR_PNQPAR5(0x1) |
                      MCF_GPIO_PNQPAR_PNQPAR3(0x1) |
                      MCF_GPIO_PNQPAR_PNQPAR2(0x1) |
                      MCF_GPIO_PNQPAR_PNQPAR1(0x1);
*/
    /* Pin assignments for port AS 
           Pin AS3 : GPIO output, initially logic '0' 
           Pin AS2 : GPIO output, initially logic '0' 
           Pin AS1 : IIC serial data, SDA 
           Pin AS0 : IIC serial clock, SCL 
    */
/*    MCF_GPIO_PORTAS = 0;
    MCF_GPIO_DDRAS = MCF_GPIO_DDRAS_DDRAS3 |
                     MCF_GPIO_DDRAS_DDRAS2;
    MCF_GPIO_PASPAR = MCF_GPIO_PASPAR_PASPAR1(0x1) |
                      MCF_GPIO_PASPAR_PASPAR0(0x1);
*/
    /* Pin assignments for port LD 
           Pin LD4 : GPIO output, initially logic '0' 
           Pin LD3 : GPIO output, initially logic '0' 
           Pin LD2 : GPIO output, initially logic '0' 
           Pin LD1 : GPIO input 
           Pin LD0 : GPIO input 
    */
    /*
    MCF_GPIO_PORTLD = 0;
    MCF_GPIO_DDRLD = MCF_GPIO_DDRLD_DDRLD3 |
                     MCF_GPIO_DDRLD_DDRLD2;*/
    //?????VT - COL LED
	    	//MCF_GPIO_DDRLD_DDRLD4 |
/*
    MCF_GPIO_PLDPAR = 0;*/

    /* Pin assignments for port QS 
           Pins are all GPIO outputs, initially all logic '0' 
    */
    /*
    MCF_GPIO_PORTQS = 0;
    MCF_GPIO_DDRQS = MCF_GPIO_DDRQS_DDRQS3 |
                     MCF_GPIO_DDRQS_DDRQS2 |
                     MCF_GPIO_DDRQS_DDRQS1 |
                     MCF_GPIO_DDRQS_DDRQS0;
    MCF_GPIO_PQSPAR = 0;*/

    /* Pin assignments for port TA 
           Pins are all GPIO outputs, initially all logic '0' 
    */
    /*
    MCF_GPIO_PORTTA = 0;
    MCF_GPIO_DDRTA = MCF_GPIO_DDRTA_DDRTA3 |
                     MCF_GPIO_DDRTA_DDRTA2 |
                     MCF_GPIO_DDRTA_DDRTA1 |
                     MCF_GPIO_DDRTA_DDRTA0;
    MCF_GPIO_PTAPAR = 0;*/

    /* Pin assignments for port TC 
           Pins are all GPIO outputs, initially all logic '0' 
    */
    /*
    MCF_GPIO_PORTTC = 0;
    MCF_GPIO_DDRTC = MCF_GPIO_DDRTC_DDRTC3 |
                     MCF_GPIO_DDRTC_DDRTC2 |
                     MCF_GPIO_DDRTC_DDRTC1 |
                     MCF_GPIO_DDRTC_DDRTC0;
    MCF_GPIO_PTCPAR = 0
        | MCF_GPIO_PTCPAR_DTIN3_GPIO
        | MCF_GPIO_PTCPAR_DTIN2_GPIO
        | MCF_GPIO_PTCPAR_DTIN1_GPIO
        | MCF_GPIO_PTCPAR_DTIN0_GPIO;
*/
//???????VT
#if 0
    /* Pin assignments for port UA 
           Pins are all GPIO inputs 
    */
    MCF_GPIO_DDRUA = 0;
    MCF_GPIO_PUAPAR = 0;
#endif

    /* Pin assignments for port UB 
           Pins are all GPIO outputs, initially all logic '0' 
    */
    //MCF_GPIO_PORTUB = 0;
    /*
    MCF_GPIO_DDRUB = MCF_GPIO_DDRUB_DDRUB3 |
                     MCF_GPIO_DDRUB_DDRUB2 |
                    // MCF_GPIO_DDRUB_DDRUB0 |
                     MCF_GPIO_DDRUB_DDRUB1;
    MCF_GPIO_PUBPAR =    MCF_GPIO_PUBPAR_PUBPAR0(0x1) ;
*/
//???????VT
#if 0
    /* Configure drive strengths */
    MCF_GPIO_PDSRH = 0;
    MCF_GPIO_PDSRL = 0;

    /* Configure Wired-OR register */
    MCF_GPIO_PWOR = 0;
#endif

#endif
    	//test_prog2();//printf("init_adc() DONE\n");

}



void start_AD( void )
{
	/* Clear stop bits */
	MCF_ADC_CTRL1 &= ~MCF_ADC_CTRL1_STOP0; 			
	MCF_ADC_CTRL2 &= ~MCF_ADC_CTRL2_STOP1; 			

	MCF_ADC_ADSDIS = 0; //MCF_ADC_ADSDIS_DS0 |MCF_ADC_ADSDIS_DS1 |MCF_ADC_ADSDIS_DS2 |MCF_ADC_ADSDIS_DS3 |
		 //MCF_ADC_ADSDIS_DS4 |MCF_ADC_ADSDIS_DS5 |MCF_ADC_ADSDIS_DS6 ;
	MCF_ADC_CAL = 0;
	//MCF_ADC_CTRL1 = 0;
	//MCF_ADC_CTRL2 = MCF_ADC_CTRL2_STOP1 | MCF_ADC_CTRL2_DIV(0x2);


	/* wait for power Up */
	MCF_ADC_POWER = MCF_ADC_POWER_PUDELAY(0xd) | MCF_ADC_POWER_PD2;

	MCF_ADC_POWER &= ~(MCF_ADC_POWER_PD0 | MCF_ADC_POWER_PD1 | MCF_ADC_POWER_PD2);

	/* wait for conv B power up */
	while (MCF_ADC_POWER & (MCF_ADC_POWER_PSTS1 | MCF_ADC_POWER_PSTS0 | MCF_ADC_POWER_PSTS2)) ;

	/* Set start bit */
	//MCF_ADC_CTRL1 = 0; 			
	MCF_ADC_CTRL2 |= MCF_ADC_CTRL2_EOSIE1 | MCF_ADC_CTRL2_SYNC1;
	MCF_ADC_CTRL1 |= MCF_ADC_CTRL1_START0; 			
	MCF_ADC_CTRL2 |= MCF_ADC_CTRL2_START1; 			

}

#endif

static void fnConfigureADC(void)
{
#if defined MJB_AN7_TEST
    ADC_SETUP adc_setup;                                                 // interrupt configuration parameters
    _CONFIG_PORT_OUTPUT(AN, (PORT_AN_BIT0 | PORT_AN_BIT1 | PORT_AN_BIT2 | PORT_AN_BIT3 | PORT_AN_BIT4 | PORT_AN_BIT5));
    adc_setup.int_type = ADC_INTERRUPT;                                  // identifier when configuring
    adc_setup.int_handler = 0;                       // handling function
    adc_setup.int_priority = 0;                           // ADC interrupt priority
    adc_setup.int_adc_bit = 7;                                           // ADC bit 0
    adc_setup.int_adc_int_type = 0; //(ADC_HIGH_LIMIT_INT);                   // interrupt types
  //adc_setup.int_adc_int_type = (ADC_END_OF_SCAN_INT | ADC_SINGLE_SHOT_TRIGGER_INT); // use to test SYNCA trigger
    adc_setup.int_adc_offset = 0;                                        // no offset
    adc_setup.int_high_level_trigger = 0; //(unsigned short)(ADC_VOLT * 2);
    adc_setup.int_adc_mode = (ADC_START_OPERATION | ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_SEQUENTIAL_MODE | ADC_SINGLE_ENDED | ADC_LOOP_MODE); // use to test single ended
  //adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_SEQUENTIAL_MODE | ADC_DIFFERENTIAL | ADC_LOOP_MODE); // use to test differential
  //adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_SEQUENTIAL_MODE | ADC_TRIGGERED_MODE); // use to test SYNCA trigger
    adc_setup.int_adc_speed = (unsigned char)(ADC_SAMPLING_SPEED(5000000));// 5MHz sampling (must be between 100kHz and 5MHz)
    adc_setup.int_adc_result = 0;                                        // no result is requested
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on high level trigger
    uTaskerMonoTimer( OWN_TASK, (DELAY_LIMIT)(3.0*SEC), E_NEXT_SAMPLE );
#elif defined MJB_AN7_TEST2
    PIT_SETUP pit_setup;                                                 // interrupt configuration parameters
//    CONFIG_TEST_OUTPUT();
    pit_setup.int_type = PIT_INTERRUPT;
    pit_setup.int_handler = test_adc;
    pit_setup.int_priority = (INTERRUPT_LEVEL_6 | INTERRUPT_PRIORITY_3);
    pit_setup.count_delay = PIT_US_DELAY(1000);                          // 1ms
    pit_setup.mode = PIT_PERIODIC;
    fnConfigureInterrupt((void *)&pit_setup);                            // enter interrupt for PIT1 test

	init_adc();
	//test_prog2();
	start_AD();
#else
    ADC_SETUP adc_setup;                                                 // interrupt configuration parameters
    adc_setup.int_type = ADC_INTERRUPT;                                  // identifier when configuring
    adc_setup.int_handler = adc_level_change_high;                       // handling function
    adc_setup.int_priority = ADC_ERR_PRIORITY;                           // ADC interrupt priority
    adc_setup.int_adc_bit = 0;                                           // ADC bit 0
    adc_setup.int_adc_int_type = (ADC_HIGH_LIMIT_INT);                   // interrupt types
  //adc_setup.int_adc_int_type = (ADC_END_OF_SCAN_INT | ADC_SINGLE_SHOT_TRIGGER_INT); // use to test SYNCA trigger
    adc_setup.int_adc_offset = 0;                                        // no offset
    adc_setup.int_high_level_trigger = (unsigned short)(ADC_VOLT * 2);
    adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_SEQUENTIAL_MODE | ADC_SINGLE_ENDED | ADC_LOOP_MODE); // use to test single ended
  //adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_SEQUENTIAL_MODE | ADC_DIFFERENTIAL | ADC_LOOP_MODE); // use to test differential
  //adc_setup.int_adc_mode = (ADC_CONFIGURE_ADC | ADC_CONFIGURE_CHANNEL | ADC_SEQUENTIAL_MODE | ADC_TRIGGERED_MODE); // use to test SYNCA trigger
    adc_setup.int_adc_speed = (unsigned char)(ADC_SAMPLING_SPEED(5000000));// 5MHz sampling (must be between 100kHz and 5MHz)
    adc_setup.int_adc_result = 0;                                        // no result is requested
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on high level trigger

  //return;                                                              // used when testing SYNCA trigger

/*  example of configuring all ADC inputs with same parameters
    adc_setup.int_adc_mode = (ADC_CONFIGURE_CHANNEL);
    adc_setup.int_adc_bit = 1;
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on high level trigger
    adc_setup.int_adc_bit = 2;
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on high level trigger
    adc_setup.int_adc_bit = 3;
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on high level trigger
    adc_setup.int_adc_bit = 4;
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on high level trigger
    adc_setup.int_adc_bit = 5;
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on high level trigger
    adc_setup.int_adc_bit = 6;
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on high level trigger
    adc_setup.int_adc_bit = 7;
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on high level trigger
*/
    adc_setup.int_adc_mode = (ADC_CONFIGURE_CHANNEL | ADC_SET_CHANNEL_OFFSET);
    adc_setup.int_adc_bit = 1;                                           // channel 1 used to test zero crossing with offset
    adc_setup.int_adc_offset = (unsigned short)(ADC_VOLT * 2.5);         // offset adjustment - for zero crossing
    adc_setup.int_adc_int_type = (ADC_ZERO_CROSSING_INT_POSITIVE | ADC_SINGLE_SHOT_CROSSING_INT); // interrupt types
    fnConfigureInterrupt((void *)&adc_setup);                            // configure test interrupt on zero crossing (2.5V)

    adc_setup.int_adc_int_type = 0;                                      // no interrupt configuration to perform
    adc_setup.int_adc_mode = (ADC_START_OPERATION);
    fnConfigureInterrupt((void *)&adc_setup);                            // start operation now
#endif
}
    #endif                                                               // end M5223X
#endif                                                                   // endif ADC configuration and interrupt handling routines




#if defined _ADC_TIMER_ROUTINES && defined TEST_PIT 
    #if defined TEST_PIT_SINGLE_SHOT
static void test_timer_int(void)
{
    static int iTimerTest = 0;
    PIT_SETUP pit_setup;                                                 // interrupt configuration parameters
    TOGGLE_TEST_OUTPUT();
    switch (iTimerTest++) {
    case 0:
        pit_setup.count_delay = PIT_MS_DELAY(1877);                      // 1.877s delay
        pit_setup.mode = PIT_SINGLE_SHOT;                                // one-shot interrupt
        break;
    case 1:
        pit_setup.count_delay = PIT_S_DELAY(5);                          // 5s periodic
        pit_setup.mode = PIT_PERIODIC;                                   // periodic interrupt
        break;
    case 6:
        iTimerTest = 0;                                                  // reset for next test
        pit_setup.mode = PIT_STOP;                                       // stop timer
        break;
    default:
        return;
    }
    pit_setup.int_type = PIT_INTERRUPT;
    pit_setup.int_handler = test_timer_int;
    pit_setup.int_priority = PIT1_INTERRUPT_PRIORITY;
    fnConfigureInterrupt((void *)&pit_setup);                            // enter interrupt for PIT1 test
}
    #elif defined TEST_PIT_PERIODIC
static void test_nmi(void)
{
    TOGGLE_TEST_OUTPUT();
}
    #endif

// PIT configuration
//
static void fnConfigurePIT(void)
{
    PIT_SETUP pit_setup;                                                 // interrupt configuration parameters
    CONFIG_TEST_OUTPUT();
    pit_setup.int_type = PIT_INTERRUPT;
    #if defined TEST_PIT_SINGLE_SHOT
    pit_setup.int_handler = test_timer_int;                              // test a single shot timer
    pit_setup.int_priority = PIT1_INTERRUPT_PRIORITY;
    pit_setup.count_delay = PIT_US_DELAY(3245);                          // 3245us delay
    pit_setup.mode = PIT_SINGLE_SHOT;                                    // one-shot interrupt
    #elif defined TEST_PIT_PERIODIC
    PORTTC |= (PORT_TC_BIT1 | PORT_TC_BIT2); DDRTC |= (PORT_TC_BIT1 | PORT_TC_BIT2);
    pit_setup.int_handler = test_nmi;
    pit_setup.int_priority = (INTERRUPT_LEVEL_7 | INTERRUPT_PRIORITY_5); // NMI to give a high resolution 50us interrupt (the interrupt may not use operating system calls!)
    pit_setup.count_delay = PIT_US_DELAY(50);                            // 50us delay
    pit_setup.mode = PIT_PERIODIC;
    #endif
    fnConfigureInterrupt((void *)&pit_setup);                            // enter interrupt for PIT1 test
}
#endif


#if defined _ADC_TIMER_ROUTINES && defined RIT_TEST                      // {7}

static void test_timer_int(void)
{
    static int iTimerTest = 0;
    RIT_SETUP rit_setup;                                                 // interrupt configuration parameters
    TOGGLE_TEST_OUTPUT();
    switch (iTimerTest++) {
    case 0:
        rit_setup.count_delay = RIT_MS_DELAY(1877);                      // 1.877s delay
        rit_setup.mode = RIT_SINGLE_SHOT;                                // one-shot interrupt
        break;
    case 1:
        rit_setup.count_delay = RIT_S_DELAY(5);                          // 5s periodic
        rit_setup.mode = RIT_PERIODIC;                                   // periodic interrupt
        break;
    case 6:
        iTimerTest = 0;                                                  // reset for next test
        rit_setup.mode = RIT_STOP;                                       // stop timer
        break;
    default:
        return;
    }
    rit_setup.int_type = RIT_INTERRUPT;
    rit_setup.int_handler = test_timer_int;
    rit_setup.int_priority = RIT_INTERRUPT_PRIORITY;
    fnConfigureInterrupt((void *)&rit_setup);                            // enter interrupt for RIT test
}

// RIT configuration
//
static void fnConfigureRIT(void)
{
    RIT_SETUP rit_setup;                                                 // interrupt configuration parameters
    CONFIG_TEST_OUTPUT();
    rit_setup.int_type = RIT_INTERRUPT;
    rit_setup.int_handler = test_timer_int;                              // test a single shot timer
    rit_setup.int_priority = RIT_INTERRUPT_PRIORITY;
    rit_setup.count_delay = RIT_US_DELAY(3245);                          // 3245us delay
    rit_setup.mode = RIT_SINGLE_SHOT;                                    // one-shot interrupt
    fnConfigureInterrupt((void *)&rit_setup);                            // enter interrupt for RIT test
}
#endif

#if defined _ADC_TIMER_ROUTINES && defined  TEST_DMA_TIMER
static void DMA_timer_int(void)
{
    static int iTimerTest = 0;
    DMA_TIMER_SETUP dma_timer_setup;                                     // interrupt configuration parameters
    TOGGLE_TEST_OUTPUT();
    switch (iTimerTest++) {
    case 0:
        dma_timer_setup.mode = (DMA_TIMER_INTERNAL_CLOCK | DMA_TIMER_SINGLE_SHOT_INTERRUPT); // one-shot interrupt
        dma_timer_setup.count_delay = DMA_TIMER_MS_DELAY(1,1,2345);      // 2.345s delay
        break;
    case 1:
        dma_timer_setup.mode = (DMA_TIMER_INTERNAL_CLOCK | DMA_TIMER_RESTART_ON_MATCH | DMA_TIMER_PERIODIC_INTERRUPT | DMA_TIMER_INTERNAL_CLOCK_DIV_16); // periodic interrupt
        dma_timer_setup.count_delay = DMA_TIMER_S_DELAY(16,1,2);         // 2s periodic
        break;
    case 6:
        iTimerTest = 0;                                                  // reset for next test
        dma_timer_setup.mode = DMA_TIMER_STOP;                           // stop timer
        break;
    default:
        return;
    }
    dma_timer_setup.channel = 1;
    dma_timer_setup.int_type = DMA_TIMER_INTERRUPT;
    dma_timer_setup.int_handler = DMA_timer_int;
    dma_timer_setup.int_priority = DMA_TIMER1_INTERRUPT_PRIORITY;
    fnConfigureInterrupt((void *)&dma_timer_setup);                      // enter interrupt for DMA timer test
    if (iTimerTest == 2) {                                               // on second test
        dma_timer_setup.channel = 2;
        dma_timer_setup.mode = (DMA_TIMER_RESTART_ON_MATCH | DMA_TIMER_TOGGLE_OUTPUT);
        dma_timer_setup.count_delay = DMA_TIMER_US_DELAY(1,1,(1000000/2/1500)); // 1500Hz signal
        fnConfigureInterrupt((void *)&dma_timer_setup);                  // generate a frequency on TOUT2
    }
}

static void fnConfigure_DMA_Timer(void)
{
    DMA_TIMER_SETUP dma_timer_setup;                                     // interrupt configuration parameters
    CONFIG_TEST_OUTPUT();
    dma_timer_setup.int_type = DMA_TIMER_INTERRUPT;
    dma_timer_setup.int_handler = DMA_timer_int;
    dma_timer_setup.channel = 1;                                         // DMA timer channel 1
    dma_timer_setup.int_priority = DMA_TIMER1_INTERRUPT_PRIORITY;        // define interrupt priority
    dma_timer_setup.mode = (DMA_TIMER_INTERNAL_CLOCK | DMA_TIMER_SINGLE_SHOT_INTERRUPT);
    dma_timer_setup.count_delay = DMA_TIMER_US_DELAY(1,1,6345);          // 6345us delay using no dividers
    fnConfigureInterrupt((void *)&dma_timer_setup);                      // enter interrupt for DMA timer test
}
#endif




#if defined _ADC_TIMER_ROUTINES && defined  TEST_GPT
// Interrupt call back from general purpose timer test
//
static void gptimer_int(void)
{
    fnInterruptMessage(OWN_TASK, CAPTURE_COMPLETE_EVENT);
}

static void fnConfigure_GPT(void)
{
    GPTIMER_SETUP gptimer_setup;                                         // interrupt configuration parameters
    gptimer_setup.int_type = GPT_TIMER_INTERRUPT;
    gptimer_setup.int_handler = gptimer_int;
    gptimer_setup.channel = 3;                                           // general purpose timer channel 3
    gptimer_setup.int_priority = GPTIMER0_INTERRUPT_PRIORITY;            // define interrupt priority
    gptimer_setup.mode = (GPT_CAPTURE_FALLING_EDGE | ENABLE_INPUT_PULLUP | GPT_INTERNAL_CLOCK | GPT_PRESCALE_16); // set up capture mode and define the timer clock
                                                                         // (in this example system clock / 2 / 16 = 1.875MHz with 60MHz: GTP timer overflows at 34.95ms but longer periods can be recorded using the extended option GPT_EXTENDED_COUNTER)
    gptimer_setup.usCaptureCount = GPT_CAPTURES;                         // request this many capture values to be recorded before calling our interrupt
    gptimer_setup.capture_list = ulCaptureList;                          // the capture list for saving to
    fnConfigureInterrupt((void *)&gptimer_setup);                        // enter interrupt for DMA timer test
}
#endif



#if defined _ADC_TIMER_ROUTINES && defined TEST_TIMER
static void timer_int(void)
{
    TOGGLE_TEST_OUTPUT();
    #if defined TEST_SINGLE_SHOT_TIMER
    fnConfigure_Timer();
    #endif
}

static void fnConfigure_Timer(void)
{
    static TIMER_INTERRUPT_SETUP timer_setup = {0};                      // interrupt configuration parameters
    CONFIG_TEST_OUTPUT();
    timer_setup.int_type = TIMER_INTERRUPT;
    timer_setup.int_priority = PRIORITY_TIMERS;
    timer_setup.int_handler = timer_int;
    timer_setup.timer_reference = 2;                                     // timer channel 2
    #ifdef _LM3SXXXX
        #if defined TEST_SINGLE_SHOT_TIMER
    timer_setup.timer_mode = (TIMER_SINGLE_SHOT | TIMER_US_VALUE/* | TIMER_16BIT_CHANNEL_B*/); // single shot timer
    timer_setup.timer_value += 100;                                      // each subsequent delay increased by 100us
        #elif defined TEST_ADC_TIMER
    timer_setup.timer_mode = (TIMER_TRIGGER_ADC | TIMER_PERIODIC | TIMER_MS_VALUE); // period timer
    timer_setup.int_handler = 0;                                         // no interrupts - used together with ADC triggering
    timer_setup.timer_value += 100;                                      // each subsequent delay increased by 100us
        #elif defined TEST_PERIODIC_TIMER
    timer_setup.timer_mode = (TIMER_PERIODIC | TIMER_MS_VALUE);          // period timer interrupt
    timer_setup.timer_value += 100;                                      // each subsequent delay increased by 100us
        #elif defined TEST_PWM                                           // {1}
            #if defined SUPPORT_PWM_CONTROLLER                           // {8}
    timer_setup.int_type    = PWM_CONFIGURATION;
    timer_setup.timer_mode  = PWM_DIV_1;                                 // don't start yet
    timer_setup.timer_value = PWM_FREQUENCY_VALUE(1000, 1);              // generate 1000Hz on timer output using PWM clock without divide
    timer_setup.pwm_value   = _PWM_PERCENT(20, PWM_FREQUENCY_VALUE(1000, 1)); // 20% PWM (high/low)
    fnConfigureInterrupt((void *)&timer_setup);                          // enter configuration for PWM test
    timer_setup.timer_reference = 5;
    timer_setup.timer_mode  = (TIMER_PWM_START_2 | TIMER_PWM_START_5 | PWM_DIV_1); // generate PWM signal on these outputs
    timer_setup.timer_value = PWM_FREQUENCY_VALUE(1500, 1);
    timer_setup.pwm_value   = _PWM_TENTH_PERCENT(706, PWM_FREQUENCY_VALUE(1500, 1)); // 70.6% PWM (high/low) on different channel
            #else
    timer_setup.timer_reference = 0;
    timer_setup.timer_mode  = (TIMER_PWM_B);                             // generate PWM signal on timer output port
    timer_setup.timer_value = TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1000));// generate 1000Hz on timer output
    timer_setup.pwm_value   = _PWM_PERCENT(20, TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1000)));// {3} 20% PWM (high/low)
    timer_setup.int_handler = 0;                                         // no interrupts
    fnConfigureInterrupt((void *)&timer_setup);                          // enter interrupt for timer test
    timer_setup.timer_mode  = (TIMER_PWM_A | TIMER_DONT_DISTURB);        // now set output A but don't disturb (reset) output B
    timer_setup.timer_value = TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1500));// generate 1500Hz on timer output
    timer_setup.pwm_value   = _PWM_TENTH_PERCENT(352, TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1500))); // 35.2% PWM (high/low)

//  example of stopping PWM outputs (first one and then complete timer module power down)
//  fnConfigureInterrupt((void *)&timer_setup);                          // enter interrupt for timer test
//  timer_setup.timer_mode  = (TIMER_STOP_PWM_B | TIMER_DONT_DISTURB);   // stop B but don't disturb A
//  fnConfigureInterrupt((void *)&timer_setup);                          // enter interrupt for timer test
//  timer_setup.timer_mode  = (TIMER_STOP_PWM_A);                        // stop A and power down timer module
            #endif
        #endif
    #elif defined _LPC23XX && defined TEST_PWM                           // {3}
    timer_setup.timer_mode  = (TIMER_PWM_MAT0);                          // generate PWM signal on MAT 2.0 output
    timer_setup.timer_value = TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1000));// generate 1000Hz on timer output
    timer_setup.pwm_value   = _PWM_PERCENT(20, TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1000))); // 20% PWM (high/low)
    fnConfigureInterrupt((void *)&timer_setup);                          // enter configuration for PWM test
    timer_setup.timer_mode  = (TIMER_PWM_MAT1 | TIMER_DONT_DISTURB);     // generate PWM signal on MAT 2.1 output without disturbing present signal
    timer_setup.pwm_value   = _PWM_TENTH_PERCENT(706, TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1000))); // 70.6% PWM (high/low) using same timer but different output
    fnConfigureInterrupt((void *)&timer_setup);                          // enter configuration for PWM test
    timer_setup.timer_reference = 1;
    timer_setup.timer_value = TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1500));// generate 1500Hz on timer output
    timer_setup.pwm_value   = _PWM_PERCENT(50, TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1500))); // 50% PWM (high/low)
    #elif (defined _LPC23XX && defined _HW_TIMER_MODE) || defined _LPC17XX || defined _STM32 || defined _KINETIS
        #if defined TEST_PERIODIC_TIMER
    timer_setup.timer_mode = (TIMER_PERIODIC);                           // period timer interrupt (activate HW timer mode if not accepted)
    timer_setup.timer_value = TIMER_MS_DELAY(1000);                      // 1s periodic interrupt
        #else
    timer_setup.timer_value += TIMER_US_DELAY(100);                      // each subsequent delay increased by 100us
        #endif
        #if defined _LPC214X
    timer_setup.timer_reference = 1;                                     // timer channel 1 since timers are limited
        #endif
    #elif defined _HW_SAM7X && defined TEST_PWM                          // {5}
    timer_setup.int_type    = PWM_CONFIGURATION;
    timer_setup.timer_mode  = (TIMER_PWM_ALT);                           // configure PWM signal on alternative PWM2 output
    timer_setup.timer_value = TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1000));// generate 1000Hz on timer output
    timer_setup.pwm_value   = _PWM_PERCENT(20, TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1000))); // 20% PWM (high/low)
    fnConfigureInterrupt((void *)&timer_setup);                          // enter configuration for PWM test
    timer_setup.timer_reference = 3;
    timer_setup.timer_mode  = (TIMER_PWM | TIMER_PWM_START_2 | TIMER_PWM_START_3); // generate PWM signal on PWM3 output and synchronise all PWM outputs
    timer_setup.timer_value = TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1500));
    timer_setup.pwm_value   = _PWM_TENTH_PERCENT(706, TIMER_US_DELAY(TIMER_FREQUENCY_VALUE(1500))); // 70.6% PWM (high/low) on different channel
    #elif defined _HW_SAM7X && defined TEST_CAPTURE
    timer_setup.timer_mode  = (TIMER_SOURCE_TIOA0);                      // configure clock source on 
    #else
        #if defined TEST_PERIODIC_TIMER
    timer_setup.timer_mode = (TIMER_PERIODIC);                           // period timer interrupt (activate HW timer mode if not accepted)
    timer_setup.timer_value = 1000;                                      // 1s periodic interrupt
        #else
    timer_setup.timer_value = 0;                                         // disable ms time delay
    timer_setup.timer_us_value += 100;                                   // each subsequent delay increased by 100us
        #endif
    #endif
    fnConfigureInterrupt((void *)&timer_setup);                          // enter interrupt for timer test
}
#endif
